@@ -8,21 +8,29 @@
 #include <R.h>
 #include <Rdefines.h>
 /*#include <Rinternals.h>*/
+#include "rgdal.h"
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 /* #include <projects.h> */
 #include <proj_api.h>
+#ifdef P4CTX
+FILE *pj_open_lib(projCtx, const char *, const char *);
+#else
 FILE *pj_open_lib(const char *, const char *);
-
+#endif
 
 SEXP
 PROJ4VersionInfo(void) {
     SEXP ans;
 
-    PROTECT(ans=NEW_CHARACTER(1));
-    SET_STRING_ELT(ans, 0, COPY_TO_USER_STRING(pj_get_release()));
+    PROTECT(ans=NEW_LIST(2));
+    SET_VECTOR_ELT(ans, 0, NEW_CHARACTER(1));
+    SET_VECTOR_ELT(ans, 1, NEW_INTEGER(1));
+    SET_STRING_ELT(VECTOR_ELT(ans, 0), 0,
+        COPY_TO_USER_STRING(pj_get_release()));
+    INTEGER_POINTER(VECTOR_ELT(ans, 1))[0] = PJ_VERSION;
 
     UNPROTECT(1);
 
@@ -39,7 +47,12 @@ PROJ4NADsInstalled(void) {
     FILE *fp;
 
     PROTECT(ans=NEW_LOGICAL(1));
+
+#ifdef P4CTX
+    fp = pj_open_lib(pj_get_default_ctx(), "conus", "rb");
+#else
     fp = pj_open_lib("conus", "rb");
+#endif
     if (fp == NULL) LOGICAL_POINTER(ans)[0] = FALSE;
     else {
         LOGICAL_POINTER(ans)[0] = TRUE;
@@ -66,8 +79,14 @@ PROJcopyEPSG(SEXP tf) {
 
 #ifdef OSGEO4W
     fp = fopen("C:\\OSGeo4W\\share\\proj\\epsg", "rb");
+#endif /* OSGEO4W */
+
+#ifndef OSGEO4W
+#ifdef P4CTX
+    fp = pj_open_lib(pj_get_default_ctx(), "epsg", "rb");
 #else
     fp = pj_open_lib("epsg", "rb");
+#endif
 #endif /* OSGEO4W */
     if (fp == NULL) INTEGER_POINTER(ans)[0] = 0;
     else {
@@ -113,7 +132,7 @@ void project(int *n, double *xlon, double *ylat, double *x, double *y, char **pr
   * using longitude and lat from xlon and ylat vectors, return
   * answers in x and y vectors (all vectors of length n) */
 
-  int i;
+  int i, nwarn=0;
 
   projUV p;
   projPJ pj;
@@ -134,12 +153,14 @@ void project(int *n, double *xlon, double *ylat, double *x, double *y, char **pr
       p.v *= DEG_TO_RAD;
       p = pj_fwd(p, pj);
       if (p.u == HUGE_VAL || ISNAN(p.u)) {
-	      Rprintf("projected point not finite\n");
+              nwarn++;
+/*	      Rprintf("projected point not finite\n");*/
       }
       x[i]=p.u;
       y[i]=p.v;
     }
   }
+  if (nwarn > 0) warning("%d projected point(s) not finite", nwarn);
 
   pj_free(pj);
 }
@@ -150,7 +171,7 @@ void project_inv(int *n, double *x, double *y, double *xlon, double *ylat, char
   * returning longitude and lat in xlon and ylat vectors, given the
   * numbers in x and y vectors (all vectors of length n) */
 
-  int i;
+  int i, nwarn=0;
 
   projUV p;
   projPJ pj;
@@ -168,12 +189,14 @@ void project_inv(int *n, double *x, double *y, double *xlon, double *ylat, char
       p.v=y[i];
       p = pj_inv(p, pj);
       if (p.u == HUGE_VAL || ISNAN(p.u)) {
-	    Rprintf("inverse projected point not finite\n");
+            nwarn++;
+/*	    Rprintf("inverse projected point not finite\n");*/
       }
       xlon[i]=p.u * RAD_TO_DEG;
       ylat[i]=p.v * RAD_TO_DEG;
     }
   }
+  if (nwarn > 0) warning("%d projected point(s) not finite", nwarn);
 
   pj_free(pj);
 }
@@ -183,7 +206,7 @@ SEXP transform(SEXP fromargs, SEXP toargs, SEXP npts, SEXP x, SEXP y) {
 	/* interface to pj_transform() to be able to use longlat proj
 	 * and datum transformation in an SEXP format */
 
-	int i, n;
+	int i, n, nwarn=0;
 	double *xx, *yy, *zz;
 	projPJ fromPJ, toPJ;
 	SEXP res;
@@ -223,8 +246,9 @@ SEXP transform(SEXP fromargs, SEXP toargs, SEXP npts, SEXP x, SEXP y) {
 
 	if( pj_transform( fromPJ, toPJ, (long) n, 0, xx, yy, zz ) != 0 ) {
 		pj_free(fromPJ); pj_free(toPJ);
-		Rprintf("error in pj_transform\n");
-		error(pj_strerrno(*pj_get_errno_ref()));
+/*		Rprintf("error in pj_transform\n"); */
+		error("error in pj_transform: %s",
+                    pj_strerrno(*pj_get_errno_ref()));
 	}
 
         pj_free(fromPJ); pj_free(toPJ);
@@ -237,12 +261,14 @@ SEXP transform(SEXP fromargs, SEXP toargs, SEXP npts, SEXP x, SEXP y) {
 	for (i=0; i < n; i++) {
 		if (xx[i] == HUGE_VAL || yy[i] == HUGE_VAL 
 		    || ISNAN(xx[i]) || ISNAN(yy[i])) {
-		    Rprintf("transformed point not finite\n");
+                    nwarn++;
+/*		    Rprintf("transformed point not finite\n");*/
 		}
 		NUMERIC_POINTER(VECTOR_ELT(res, 0))[i] = xx[i];
 		NUMERIC_POINTER(VECTOR_ELT(res, 1))[i] = yy[i];
 	}
 
+        if (nwarn > 0) warning("%d projected point(s) not finite", nwarn);
 	UNPROTECT(1);
 	return(res);
 }
